#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "kpf_io.h"

// KnotPlot 1.0 data files are stored using 
// IRIX byte order ("big endian")

void read_unsigned_short (unsigned short *v, FILE *fp) {
  unsigned char uca, ucb;
  fread (&ucb, 1, 1, fp);
  fread (&uca, 1, 1, fp);
  *v = (ucb << 8) | uca;
}

void read_long (long *v, FILE *fp) {
  unsigned char uc;
  *v = 0;
  for (int shift = 24; shift >= 0; shift -= 8) {
    fread (&uc, 1, 1, fp);
    *v |= uc << shift;
  }
}

void read_float (float *v, FILE *fp) {
  union {
    long lg;
    float flt;
  } x;

  read_long (&x.lg, fp);
  *v = x.flt;
}

int needs_swapping () {
  // returns TRUE on a "little endian" machine (Intel)
  // returns FALSE on a "big endian" machine (IRIX, Solaris)
  
  short super_gumby = 0x0001;
  char *swap_dat_thing = (char *) &super_gumby;
  return (int) *swap_dat_thing;
}

void read_double (double *v, FILE *fp) {
  unsigned char *ucp;
  ucp = (unsigned char *) v;

  if (needs_swapping ()) {
    for (int i = 0; i < 8; i++)
      fread (ucp + 7 - i, 1, 1, fp);
  }
  else {
    for (int i = 0; i < 8; i++)
      fread (ucp++, 1, 1, fp);
  }
}

void read_fvector (fvector v, FILE *fp) {
  read_float (v, fp);
  read_float (v + 1, fp);
  read_float (v + 2, fp);
}
